import os
from statistics import mode
import pandas as pd
import re
import sys
import splitfolders
import spacy_udpipe


spacy_udpipe.download("cs")
nlp = spacy_udpipe.load("cs")


"""
Sort names of docs (same for UNIX and WINDOWS)

Arguments:
    data: List to sort
"""
def sorted_alphanumeric(data):
    convert = lambda text: int(text) if text.isdigit() else text.lower()
    alphanum_key = lambda key: [ convert(c) for c in re.split('([0-9]+)', key) ] 
    return sorted(data, key=alphanum_key)

"""
Creates a dataframe with a structure like WORD (every word of each article) in first column and KEYWORDS (0 / 1) in second according to anotators

Arguments:
    folder_docs: Path to folder with documents
    folder_keys: Path to folder with keywords
    docs: list of documents file names
    keys: list of keywords file names

Returns:
    d: dataframe
"""
def assign_kw(folder_docs, folder_keys, docs, keys):
    words_array = []
    for i in range(0, len(docs)):
        file = open(folder_docs + docs[i], encoding="utf8")
        line = file.read().replace("\n", " ").lower()
        words = re.sub("[^\w]", " ",  line).split()
        words.append("###")
        file.close()
        words_array.append(words)

    
    assigned_kw = []
    for i in range(0, len(keys)):
        file = open(folder_keys + keys[i], encoding="utf8")
        assigned_kw.append(file.read().split())
        file.close()

    
    labels = []
    tmp = []
    for i in range(0, len(words_array)):
        kw_nlp = nlp(assigned_kw[i])
        lemma_kw = []
        for k in kw_nlp:
            lemma_kw.append(k.lemma_)
        for j in range(0, len(words_array[i])):
            w_nlp = nlp(words_array[i][j])
            lemma_w = []
            for k in w_nlp:
                lemma_w.append(k.lemma_)
            if(any(item in lemma_w for item in lemma_kw)):
                tmp.append(1)
            elif(words_array[i][j] == "###"):
                tmp.append("###")
            else:
                tmp.append(0)
        labels.append(tmp)
        tmp = []


    data_tuples = list(zip(words_array, labels))
    d = pd.DataFrame(data_tuples, columns=['article','label'])
    return d


"""
Writes to evaluation data to file

Arguments:
    data: dataframe with structure like WORD | KEYWORD
    dataset: dataset name
"""
def write_to_eval_file(data, dataset):
    articles = data['article']
    keywords = data['label']
    for i in range(0, len(articles)):
        data_tuples = list(zip(articles[i], keywords[i]))
        new_df = pd.DataFrame(data_tuples, columns=['WORD', 'KEYWORD'])
        new_df.to_csv(dataset+"/eval.tsv", sep="\t", index=False, mode='a')


"""
Split dataset to Train-Test and create evaluation file on Test part
"""
if __name__ == '__main__':
    if(len(sys.argv) < 2):
        print("Please specify path to dataset")
        exit()

    if os.path.isdir(sys.argv[1]+"docs") and os.path.isdir(sys.argv[1]+"keys"):
        dataset = sys.argv[1]
    else:
        print("There is no suitable dataset!")
        exit()
     
    new_dataset = dataset[:-1] + "_SPLIT"

    if os.path.isdir(new_dataset):
        print("Splitted datased already exists!\n")
        exit()

    print("Creating Train/Test corpus")
    splitfolders.ratio(dataset, output=new_dataset , seed=1337, ratio=(.8, 0.2))
    docs_names = sorted_alphanumeric(os.listdir(new_dataset+"/val/docs"))
    keys_names = sorted_alphanumeric(os.listdir(new_dataset+"/val/keys"))

    data = assign_kw(new_dataset + "/val/docs/", new_dataset+"/val/keys/",docs_names, keys_names)
    write_to_eval_file(data, new_dataset)