from cgi import print_environ
import sys
import os
import pandas as pd
from sklearn.metrics import accuracy_score, precision_score, recall_score, f1_score


"""
Computes metrics (accuracy, precission, recall, f score)

Arguments:
    true: true labels
    pred: predicted labels by method

Returns:
    accuracy: accuracy
    precission: precission
    recall: recall
    f_score: f score
"""
def evaluation(true, pred):
    to_remove = ["###", "WORD", "KEYWORD"]
    true = [int(x) for x in true if (x not in to_remove)]
    pred = [int(x) for x in pred if (x not in to_remove)]

    accuracy = accuracy_score(true, pred)
    precission = precision_score(true, pred)
    recall = recall_score(true, pred)
    f_score = 2 * (precission * recall) / (precission + recall)
    print("Accuracy: {}".format(accuracy))
    print("Precission: {}".format(precission))
    print("Recall: {}".format(recall))
    print("F score: {}".format(f_score))

    return accuracy, precission, recall, f_score


"""
Computes metrics based on two files (evaluation file created on test dataset and evaluation file created by method)
"""
if __name__ == '__main__':
    if(len(sys.argv) < 3):
        print("Please specify:\narg1: path to evaluation file\narg2: path to predictions file")
        exit()

    eval_file = sys.argv[1]
    pred_file = sys.argv[2]

    eval_df = pd.read_csv(eval_file, sep='\t')
    pred_df = pd.read_csv(pred_file, sep='\t')

    eval_vector = eval_df['KEYWORD'].tolist()
    pred_vector = pred_df['KEYWORD'].tolist()

    if(len(eval_vector) != len(pred_vector)):
        print("Evaluation files does not have the same length!\n")
        exit()


    accuracy, precission, recall, f_score = evaluation(eval_vector, pred_vector)
    
