from re import T
import os, sys
from string import digits
import pandas as pd
import re
import textrank
import yake_method
import bert_method
import evaluation
import tf_idf

from corpy.udpipe import Model

import wandb
WANDB_API_KEY= "8925984edb6fe740e4626468d3584717a960537f"

METHODS = [tf_idf, textrank, yake_method, bert_method]
methods_name = ['TF-IDF', 'TextRank', 'YAKE!', 'KeyBert']
#wandb.login(key=WANDB_API_KEY)

#wandb.init(project="keyword extraction")


"""
Sort names of docs (same for UNIX and WINDOWS)

Arguments:
    data: List to sort
"""
def sorted_alphanumeric(data):
    convert = lambda text: int(text) if text.isdigit() else text.lower()
    alphanum_key = lambda key: [ convert(c) for c in re.split('([0-9]+)', key) ] 
    return sorted(data, key=alphanum_key)

"""
Gets articles and keywords file names

Arguments:
    docs: path to articles folder
    keys: path to keywords folder

Returns:
    docs_names: list of articles file names
    keys_names: list of keywords file names
"""
def prepare_articles(docs, keys):
    # read folder
    docs_names = sorted_alphanumeric(os.listdir(docs))
    keys_names = sorted_alphanumeric(os.listdir(keys))
    return docs_names, keys_names
    

"""
Creates a eval file

Arguments: 
    dataset: path to dataset
    keywords: keywords gets by method
    article: list of articles
    name: method name
"""
def save_to_file(dataset, keywords, article, name):
    lens = []
    words_array = []
    counter = 0
    for art in article:
        words = re.sub("[^\w]", " ",  art).split()
        counter+=1
        lens.append(len(words))
        words.append("###")
        words_array.append(words)

    labels = []
    tmp = []
    for i in range(0, len(words_array)):
        for j in range(0, len(words_array[i])):
            if(words_array[i][j] in keywords[i]):
                tmp.append(1)
            elif(words_array[i][j] == "###"):
                tmp.append("###")
            else:
                tmp.append(0)
        labels.append(tmp)
        tmp = []

    data_tuples = list(zip(words_array, labels))
    d = pd.DataFrame(data_tuples, columns=['article','label'])
    articles = d['article']
    keywords = d['label']

    if os.path.exists(name + "_eval.tsv") == True:
        os.remove(name + "_eval.tsv")

    for i in range(0, len(articles)):
        data_tuples = list(zip(articles[i], keywords[i]))
        new_df = pd.DataFrame(data_tuples, columns=['WORD', 'KEYWORD'])
        new_df.to_csv(name + "_eval.tsv", sep="\t", index=False, mode='a')

    if not os.path.exists(dataset + "/eval.tsv"):
        print("WARNING: Evaluation file does not exists!")
        return None, None, None, None

    eval_file = dataset + "/eval.tsv"
    eval_df = pd.read_csv(eval_file, sep='\t')
    eval_vector = eval_df['KEYWORD'].tolist()

    pred_file = name + "_eval.tsv"
    pred_df = pd.read_csv(pred_file, sep='\t')
    pred_vector = pred_df['KEYWORD'].tolist()
    a, p, r, f = evaluation.evaluation(eval_vector, pred_vector)
    return a, p, r, f


"""
Copmutes metrics for all baseline methods 

Arguments: 
    dataset: Path to dataset
    d_folder: folder with articles files
    k_folder: folder with keywords files
    docs_names: list of articles file names
    keys_names: list of keywords file names 
"""
def get_keywords(dataset, d_folder, k_folder, docs_names, keys_names):
    name = 0
    assigned_kw = []
    for i in range(0, len(keys_names)):
        file = open(k_folder + keys_names[i], encoding="utf8")
        assigned_kw.append(file.read().split())
        file.close()

    for method in METHODS:
        wandb.init(project='keyword extraction', entity='jkrajnak')
        wandb.run.name = "Eval_baseline"
        config = wandb.config

        text_array = []
        for i in range(0, len(docs_names)):
            file = open(d_folder + docs_names[i], encoding="utf8")
            line = file.read().replace("\n", " ")
            file.close()
            text_array.append(line)

        # method to use
        print(f"{str(methods_name[name])}: getting keywords...")
        if str(methods_name[name]) == "TextRank":
            for i in range(0, len(assigned_kw)):
                assigned_kw[i] = assigned_kw[i][:100]

        lan = "en"
        if "CZ" in dataset:
            lan = "cz"

        keywords = method.get_keywords(text_array, lan, 1, assigned_kw)

        a, p, r, f = save_to_file(dataset, keywords, text_array, methods_name[name])
        
        wandb.log({"Dataset": d_folder})
        wandb.log({"Method": methods_name[name]})
        wandb.log({"Acuraccy": a})
        wandb.log({"F_score": f})
        wandb.log({"Precission": p})
        wandb.log({"Recall": r})
        wandb.finish(None)
        name += 1


"""
Computes metrics for all baseline methods (TF-IDF, TextRank, YAKE!, KeyBert)
"""
if __name__ == '__main__':
    if len(sys.argv) < 2:
        print("Please specify dataset")
        exit()


    if os.path.isdir(sys.argv[1]+"val/docs") and os.path.isdir(sys.argv[1]+"val/keys"):
        dataset_docs = sys.argv[1] + "val/docs/"
        dataset_keys = sys.argv[1] + "val/keys/"

    elif os.path.isdir(sys.argv[1]+"docs") and os.path.isdir(sys.argv[1]+"/keys"):
        print("WARNING: dataset has no evaluation file, metrics will not be computed!")
        dataset_docs = sys.argv[1] + "docs/"
        dataset_keys = sys.argv[1] + "keys/"
    else:
        print("There is no suitable dataset!")
        exit()

    docs_names, keys_names = prepare_articles(dataset_docs, dataset_keys)

    get_keywords(sys.argv[1], dataset_docs, dataset_keys, docs_names, keys_names)