from errno import ECHILD
import os, sys
from pydoc import doc
import json
import itertools
from dominate.tags import *
from torch import le



"""
Loads data from input jsonl files

Arguments:
    docs: path to a file with documents
    keys: path to a file with keywords

Returns:
    articles: list of articles
    json_data: list of all keywords
"""
def load_data(docs, keys):
    
    with open(docs, encoding='utf8') as f:
        articles = f.read().split("]")
        
    for i in range(0, len(articles)):
        articles[i] = articles[i].replace("[", "").replace("\"", "").split()


    with open(keys, 'r', encoding='utf8') as handle:
        json_data = [json.loads(line) for line in handle]

    
    return articles, json_data
 

"""
Create a "more readable" list

Arguments:
    keywords: list of keywords

Returns:
    kw_list: list of keywords
"""
def create_lists(keywords):
    article_counter = 0
    kw_list = []
    kw_art_list = []
    kw_an_list = []
    for art in keywords:
        kw_an_list = []
        for anotator in art.keys():
            kw_art_list = []
            for kw in art[anotator].keys():
                kw_art_list.append(art[anotator][kw])
                
            kw_art_list = (list(itertools.chain.from_iterable(kw_art_list)))
            kw_an_list.append(kw_art_list)
        kw_list.append(kw_an_list)
        article_counter += 1

    return kw_list


"""
Get counts of occurences keyword based on how many anotators mark the word as a keyword

Arguments:
    articles: list of articles
    keywords: list of anotated keywords

Returns: list of count of occurences
"""
def get_occurences(articles, keywords):
    occurences = []

    for art in range(0, len(keywords)):
        occurences_art = [0] * len(articles[art])
        for i in range(0, len(articles[art])):
            for j in range(0, len(keywords[art])):
                clear_w = articles[art][i].replace(".","").replace(",","")
                if(clear_w in keywords[art][j]):
                    occurences_art[i] += 1
        occurences.append(occurences_art)

    return occurences


"""
Create a HTML file

Arguments:
    occur: list of counts of occurences keyword based on how many anotators mark the word as a keyword 
    kw: list of keywords
    art: list of articles
    to_show: determine which article will be shown
"""
def create_html(occur, kw, art, to_show):
    for i in range(0, len(occur)):
        anotator_count = len(kw[i])
        for j in range(0, len(occur[i])):
            if occur[i][j] / anotator_count >= 0.8:
                art[i][j] = "<span style='background-color: #ff5200 '>" + art[i][j] + "</span>"
            elif occur[i][j] / anotator_count >= 0.6:
                art[i][j] = "<span style='background-color: #ff8a1d'>" + art[i][j] + "</span>"
            elif occur[i][j] / anotator_count >= 0.4:
                art[i][j] = "<span style='background-color: #ffd558'>" + art[i][j] + "</span>"
            elif occur[i][j] / anotator_count >= 0.2:
                art[i][j] = "<span style='background-color: #fffeac'>" + art[i][j] + "</span>"
        art[i] = ' '.join(art[i])        

    
    html_string = f"""<p>{art[to_show]}</p> <br> <H1>Legenda</H1>procento anotátorů, kteří označili dané slovo jako klíčové <br> 
    <span style='background-color: #ff5200'> >= 80% </span>
    <span style='background-color: #ff8a1d'> >= 60% </span>
    <span style='background-color: #ffd558'> >= 40% </span>
    <span style='background-color: #fffeac'> >= 20% </span>
    """

    f = open("mark_kw.html", "w")
    f.write(html_string)
    f.close()


"""
Create a HTML file with marked keyword for article specified in command line
"""
if __name__ == '__main__':
    if len(sys.argv) != 4:
        print("Please specify:\narg1: path to file with docs\narg2: path to file with keys\narg3: index of doc to show")
        exit()

    dataset_docs = sys.argv[1]
    dataset_keys = sys.argv[2]
    art_to_show = sys.argv[3]

    #dataset_docs = "CZ_data_v2/summaries.jsonl"
    #dataset_keys = "CZ_data_v2/queries.jsonl"
    
    articles, keywords = load_data(dataset_docs, dataset_keys)

    
    if(int(art_to_show) >= len(articles)-1):
        print(f"Plese specify valid document index in range (0 - {len(keywords) - 1})")
        exit()

    if(len(articles) - 1 != len(keywords)):
        print("Input files does not match!")
        exit()
    
    
    kw_list = create_lists(keywords)

    occurences = get_occurences(articles, kw_list)

    create_html(occurences, kw_list, articles, int(art_to_show))